import numpy as np
from scipy.interpolate import griddata
import matplotlib.pyplot as plt
import numpy.ma as ma
from numpy.random import uniform, seed
import os
from matplotlib.colors import hsv_to_rgb
import colorsys
from skimage import measure
import matplotlib.ticker as ticker
import matplotlib.font_manager as fm
from colour import Color
import matplotlib.gridspec as gridspec
import math
from matplotlib.pyplot import figure, show, grid, tight_layout
from PIL import Image
import scipy.ndimage
import matplotlib.image as mpimg
import matplotlib.patches as patches




"""
======================
Definitions
======================
"""
def cm2inch(value):
    return value/2.54

Aalto_yellow = '#FECB00' 
Aalto_orange = '#FF7900' #
Aalto_red = '#ED2939' 
Aalto_violet = '#B1059D' #
Aalto_purple = '#6639B7'
Aalto_blue = '#0065BD' #
Aalto_turqoise = '#00A8B4'
Aalto_green = '#009B3A' #
Aalto_light_green = '#69BE28'
Aalto_grey = '#928B81'

nmpp = 21.6 #This is nm per image pixel

Anisotropy = 1e4 # J/m^3
Exchange = 3.1e-11  # J/m
SatMag = 1.46e6  # A/m
mu0 = 4*np.pi*1e-7

var_width_factor = 2

"""
2. define parameters:
"""

headerrows = 3


label_fontsize = 8
tick_fontsize = 6

contourlinewidth = 0.5
plotlinewidth = 1
symbolsize = 3



# define panel size and position:
width = 8.6
heightratio = 0.45

x1 = 0.11
x2 = 0.51

y1 = 0.17
y2 = y1




width1 = 0.38
height1 = 0.82

figuresize=(cm2inch(width), cm2inch(heightratio*width))

panel_1 = [x1, y1, width1, height1]
panel_2 = [x2, y2, width1, height1]


rlabelpad = -1
thetalabelpad = -3
xlabelpad = 1
ylabelpad = 1

xScaling = 1e3

"""
create image:
"""
fig = plt.figure(num=None, figsize=figuresize, dpi=80, facecolor='w', edgecolor='k')

ax1 = fig.add_axes(panel_1, label ='1')


ax2 = fig.add_axes(panel_2, label ='2')

"""
======================
import data
======================
"""


DW_2p5um_Ku1e4_LC = np.loadtxt('t-dep_2p5um/LC/analysis/DW_data_LC.txt',skiprows=2)
DW_2p5um_Ku1e4_LU = np.loadtxt('t-dep_2p5um/LU/analysis/DW_data_LU.txt',skiprows=2)
DW_2p5um_Ku1e4_SC = np.loadtxt('t-dep_2p5um/SC/analysis/DW_data_SC.txt',skiprows=2)
DW_2p5um_Ku1e4_SU = np.loadtxt('t-dep_2p5um/SU/analysis/DW_data_SU.txt',skiprows=2)

Energy_2p5um_Ku1e4_LC = np.loadtxt('t-dep_2p5um/LC/analysis/combined_odts_LC.txt',skiprows=2)
Energy_2p5um_Ku1e4_LU = np.loadtxt('t-dep_2p5um/LU/analysis/combined_odts_LU.txt',skiprows=2)
Energy_2p5um_Ku1e4_SC = np.loadtxt('t-dep_2p5um/SC/analysis/combined_odts_SC.txt',skiprows=2)
Energy_2p5um_Ku1e4_SU = np.loadtxt('t-dep_2p5um/SU/analysis/combined_odts_SU.txt',skiprows=2)


"""
======================
Calculate Energy densities
======================
"""
# E_tot = column 1
# E_exch = column 6
# E_demag = column 7
# E_ani = column 9

eDens_2p5um_Ku1e4_LC = np.zeros((Energy_2p5um_Ku1e4_LC.shape[0],5))
eDens_2p5um_Ku1e4_LU = np.zeros((Energy_2p5um_Ku1e4_LU.shape[0],5))
eDens_2p5um_Ku1e4_SC = np.zeros((Energy_2p5um_Ku1e4_SC.shape[0],5))
eDens_2p5um_Ku1e4_SU = np.zeros((Energy_2p5um_Ku1e4_SU.shape[0],5))


eDens_2p5um_Ku1e4_LC[:,0] = Energy_2p5um_Ku1e4_LC[:,0]
eDens_2p5um_Ku1e4_LU[:,0] = Energy_2p5um_Ku1e4_LU[:,0]
eDens_2p5um_Ku1e4_SC[:,0] = Energy_2p5um_Ku1e4_SC[:,0]
eDens_2p5um_Ku1e4_SU[:,0] = Energy_2p5um_Ku1e4_SU[:,0]

#e_tot to column 1 form E_tot in column 1
eDens_2p5um_Ku1e4_LC[:,1] = Energy_2p5um_Ku1e4_LC[:,1]/(1e-6*8e-6*Energy_2p5um_Ku1e4_LC[:,0]*1e-9)
eDens_2p5um_Ku1e4_LU[:,1] = Energy_2p5um_Ku1e4_LU[:,1]/(1e-6*8e-6*Energy_2p5um_Ku1e4_LU[:,0]*1e-9)
eDens_2p5um_Ku1e4_SC[:,1] = Energy_2p5um_Ku1e4_SC[:,1]/(1e-6*8e-6*Energy_2p5um_Ku1e4_SC[:,0]*1e-9)
eDens_2p5um_Ku1e4_SU[:,1] = Energy_2p5um_Ku1e4_SU[:,1]/(1e-6*8e-6*Energy_2p5um_Ku1e4_SU[:,0]*1e-9)

#e_exch to column 2 form E_exch in column 6
eDens_2p5um_Ku1e4_LC[:,2] = Energy_2p5um_Ku1e4_LC[:,6]/(1e-6*8e-6*Energy_2p5um_Ku1e4_LC[:,0]*1e-9)
eDens_2p5um_Ku1e4_LU[:,2] = Energy_2p5um_Ku1e4_LU[:,6]/(1e-6*8e-6*Energy_2p5um_Ku1e4_LU[:,0]*1e-9)
eDens_2p5um_Ku1e4_SC[:,2] = Energy_2p5um_Ku1e4_SC[:,6]/(1e-6*8e-6*Energy_2p5um_Ku1e4_SC[:,0]*1e-9)
eDens_2p5um_Ku1e4_SU[:,2] = Energy_2p5um_Ku1e4_SU[:,6]/(1e-6*8e-6*Energy_2p5um_Ku1e4_SU[:,0]*1e-9)

#e_demag to column 3 form E_demag in column 7
eDens_2p5um_Ku1e4_LC[:,3] = Energy_2p5um_Ku1e4_LC[:,7]/(1e-6*8e-6*Energy_2p5um_Ku1e4_LC[:,0]*1e-9)
eDens_2p5um_Ku1e4_LU[:,3] = Energy_2p5um_Ku1e4_LU[:,7]/(1e-6*8e-6*Energy_2p5um_Ku1e4_LU[:,0]*1e-9)
eDens_2p5um_Ku1e4_SC[:,3] = Energy_2p5um_Ku1e4_SC[:,7]/(1e-6*8e-6*Energy_2p5um_Ku1e4_SC[:,0]*1e-9)
eDens_2p5um_Ku1e4_SU[:,3] = Energy_2p5um_Ku1e4_SU[:,7]/(1e-6*8e-6*Energy_2p5um_Ku1e4_SU[:,0]*1e-9)

#e_ani to column 4 form E_ani in column 9
eDens_2p5um_Ku1e4_LC[:,4] = Energy_2p5um_Ku1e4_LC[:,9]/(1e-6*8e-6*Energy_2p5um_Ku1e4_LC[:,0]*1e-9)
eDens_2p5um_Ku1e4_LU[:,4] = Energy_2p5um_Ku1e4_LU[:,9]/(1e-6*8e-6*Energy_2p5um_Ku1e4_LU[:,0]*1e-9)
eDens_2p5um_Ku1e4_SC[:,4] = Energy_2p5um_Ku1e4_SC[:,9]/(1e-6*8e-6*Energy_2p5um_Ku1e4_SC[:,0]*1e-9)
eDens_2p5um_Ku1e4_SU[:,4] = Energy_2p5um_Ku1e4_SU[:,9]/(1e-6*8e-6*Energy_2p5um_Ku1e4_SU[:,0]*1e-9)

# calculate energy differences

DeltaSCLU_2p5um = np.zeros((Energy_2p5um_Ku1e4_LC.shape[0],5))
DeltaLCSU_2p5um = np.zeros((Energy_2p5um_Ku1e4_LU.shape[0],5))
DeltaSCSU_2p5um = np.zeros((Energy_2p5um_Ku1e4_SC.shape[0],5))
DeltaLCLU_2p5um = np.zeros((Energy_2p5um_Ku1e4_SU.shape[0],5))

DeltaSCLU_2p5um[:,0] = eDens_2p5um_Ku1e4_LC[:,0]
DeltaLCSU_2p5um[:,0] = eDens_2p5um_Ku1e4_LU[:,0]
DeltaSCSU_2p5um[:,0] = eDens_2p5um_Ku1e4_SC[:,0]
DeltaLCLU_2p5um[:,0] = eDens_2p5um_Ku1e4_SU[:,0]

DeltaSCLU_2p5um[:,1] = 2*(eDens_2p5um_Ku1e4_SC[:,1]-eDens_2p5um_Ku1e4_LU[:,1])/(eDens_2p5um_Ku1e4_SC[:,1]+eDens_2p5um_Ku1e4_LU[:,1])*100
DeltaLCSU_2p5um[:,1] = 2*(eDens_2p5um_Ku1e4_LC[:,1]-eDens_2p5um_Ku1e4_SU[:,1])/(eDens_2p5um_Ku1e4_LC[:,1]+eDens_2p5um_Ku1e4_SU[:,1])*100
DeltaSCSU_2p5um[:,1] = 2*(eDens_2p5um_Ku1e4_SC[:,1]-eDens_2p5um_Ku1e4_SU[:,1])/(eDens_2p5um_Ku1e4_SC[:,1]+eDens_2p5um_Ku1e4_SU[:,1])*100
DeltaLCLU_2p5um[:,1] = 2*(eDens_2p5um_Ku1e4_LC[:,1]-eDens_2p5um_Ku1e4_LU[:,1])/(eDens_2p5um_Ku1e4_LC[:,1]+eDens_2p5um_Ku1e4_LU[:,1])*100

DeltaSCLU_2p5um[:,2] = 2*(eDens_2p5um_Ku1e4_SC[:,2]-eDens_2p5um_Ku1e4_LU[:,2])/(eDens_2p5um_Ku1e4_SC[:,2]+eDens_2p5um_Ku1e4_LU[:,2])*100
DeltaLCSU_2p5um[:,2] = 2*(eDens_2p5um_Ku1e4_LC[:,2]-eDens_2p5um_Ku1e4_SU[:,2])/(eDens_2p5um_Ku1e4_LC[:,2]+eDens_2p5um_Ku1e4_SU[:,2])*100
DeltaSCSU_2p5um[:,2] = 2*(eDens_2p5um_Ku1e4_SC[:,2]-eDens_2p5um_Ku1e4_SU[:,2])/(eDens_2p5um_Ku1e4_SC[:,2]+eDens_2p5um_Ku1e4_SU[:,2])*100
DeltaLCLU_2p5um[:,2] = 2*(eDens_2p5um_Ku1e4_LC[:,2]-eDens_2p5um_Ku1e4_LU[:,2])/(eDens_2p5um_Ku1e4_LC[:,2]+eDens_2p5um_Ku1e4_LU[:,2])*100

DeltaSCLU_2p5um[:,3] = 2*(eDens_2p5um_Ku1e4_SC[:,3]-eDens_2p5um_Ku1e4_LU[:,3])/(eDens_2p5um_Ku1e4_SC[:,3]+eDens_2p5um_Ku1e4_LU[:,3])*100
DeltaLCSU_2p5um[:,3] = 2*(eDens_2p5um_Ku1e4_LC[:,3]-eDens_2p5um_Ku1e4_SU[:,3])/(eDens_2p5um_Ku1e4_LC[:,3]+eDens_2p5um_Ku1e4_SU[:,3])*100
DeltaSCSU_2p5um[:,3] = 2*(eDens_2p5um_Ku1e4_SC[:,3]-eDens_2p5um_Ku1e4_SU[:,3])/(eDens_2p5um_Ku1e4_SC[:,3]+eDens_2p5um_Ku1e4_SU[:,3])*100
DeltaLCLU_2p5um[:,3] = 2*(eDens_2p5um_Ku1e4_LC[:,3]-eDens_2p5um_Ku1e4_LU[:,3])/(eDens_2p5um_Ku1e4_LC[:,3]+eDens_2p5um_Ku1e4_LU[:,3])*100

DeltaSCLU_2p5um[:,4] = 2*(eDens_2p5um_Ku1e4_SC[:,4]-eDens_2p5um_Ku1e4_LU[:,4])/(eDens_2p5um_Ku1e4_SC[:,4]+eDens_2p5um_Ku1e4_LU[:,4])*100
DeltaLCSU_2p5um[:,4] = 2*(eDens_2p5um_Ku1e4_LC[:,4]-eDens_2p5um_Ku1e4_SU[:,4])/(eDens_2p5um_Ku1e4_LC[:,4]+eDens_2p5um_Ku1e4_SU[:,4])*100
DeltaSCSU_2p5um[:,4] = 2*(eDens_2p5um_Ku1e4_SC[:,4]-eDens_2p5um_Ku1e4_SU[:,4])/(eDens_2p5um_Ku1e4_SC[:,4]+eDens_2p5um_Ku1e4_SU[:,4])*100
DeltaLCLU_2p5um[:,4] = 2*(eDens_2p5um_Ku1e4_LC[:,4]-eDens_2p5um_Ku1e4_LU[:,4])/(eDens_2p5um_Ku1e4_LC[:,4]+eDens_2p5um_Ku1e4_LU[:,4])*100

# calculate energy ratios

RatioSCLU_2p5um = np.zeros((Energy_2p5um_Ku1e4_LC.shape[0],5))
RatioLCSU_2p5um = np.zeros((Energy_2p5um_Ku1e4_LU.shape[0],5))
RatioSCSU_2p5um = np.zeros((Energy_2p5um_Ku1e4_SC.shape[0],5))
RatioLCLU_2p5um = np.zeros((Energy_2p5um_Ku1e4_SU.shape[0],5))

RatioSCLU_2p5um[:,0] = eDens_2p5um_Ku1e4_LC[:,0]
RatioLCSU_2p5um[:,0] = eDens_2p5um_Ku1e4_LU[:,0]
RatioSCSU_2p5um[:,0] = eDens_2p5um_Ku1e4_SC[:,0]
RatioLCLU_2p5um[:,0] = eDens_2p5um_Ku1e4_SU[:,0]

RatioSCLU_2p5um[:,1] = eDens_2p5um_Ku1e4_SC[:,1]/eDens_2p5um_Ku1e4_LU[:,1]
RatioLCSU_2p5um[:,1] = eDens_2p5um_Ku1e4_LC[:,1]/eDens_2p5um_Ku1e4_SU[:,1]
RatioSCSU_2p5um[:,1] = eDens_2p5um_Ku1e4_SC[:,1]/eDens_2p5um_Ku1e4_SU[:,1]
RatioLCLU_2p5um[:,1] = eDens_2p5um_Ku1e4_LC[:,1]/eDens_2p5um_Ku1e4_LU[:,1]

RatioSCLU_2p5um[:,2] = eDens_2p5um_Ku1e4_SC[:,2]/eDens_2p5um_Ku1e4_LU[:,2]
RatioLCSU_2p5um[:,2] = eDens_2p5um_Ku1e4_LC[:,2]/eDens_2p5um_Ku1e4_SU[:,2]
RatioSCSU_2p5um[:,2] = eDens_2p5um_Ku1e4_SC[:,2]/eDens_2p5um_Ku1e4_SU[:,2]
RatioLCLU_2p5um[:,2] = eDens_2p5um_Ku1e4_LC[:,2]/eDens_2p5um_Ku1e4_LU[:,2]

RatioSCLU_2p5um[:,3] = eDens_2p5um_Ku1e4_SC[:,3]/eDens_2p5um_Ku1e4_LU[:,3]
RatioLCSU_2p5um[:,3] = eDens_2p5um_Ku1e4_LC[:,3]/eDens_2p5um_Ku1e4_SU[:,3]
RatioSCSU_2p5um[:,3] = eDens_2p5um_Ku1e4_SC[:,3]/eDens_2p5um_Ku1e4_SU[:,3]
RatioLCLU_2p5um[:,3] = eDens_2p5um_Ku1e4_LC[:,3]/eDens_2p5um_Ku1e4_LU[:,3]

RatioSCLU_2p5um[:,4] = eDens_2p5um_Ku1e4_SC[:,4]/eDens_2p5um_Ku1e4_LU[:,4]
RatioLCSU_2p5um[:,4] = eDens_2p5um_Ku1e4_LC[:,4]/eDens_2p5um_Ku1e4_SU[:,4]
RatioSCSU_2p5um[:,4] = eDens_2p5um_Ku1e4_SC[:,4]/eDens_2p5um_Ku1e4_SU[:,4]
RatioLCLU_2p5um[:,4] = eDens_2p5um_Ku1e4_LC[:,4]/eDens_2p5um_Ku1e4_LU[:,4]



print('120C/60U total from', *RatioLCSU_2p5um[np.where(RatioLCSU_2p5um[:,0]==2)[0],1], 'to', *RatioLCSU_2p5um[np.where(RatioLCSU_2p5um[:,0]==20)[0],1])
print('60C/120U total from', *RatioSCLU_2p5um[np.where(RatioSCLU_2p5um[:,0]==2)[0],1], 'to', *RatioSCLU_2p5um[np.where(RatioSCLU_2p5um[:,0]==20)[0],1])
print('60C/60U total from', *RatioSCSU_2p5um[np.where(RatioSCSU_2p5um[:,0]==2)[0],1], 'to', *RatioSCSU_2p5um[np.where(RatioSCSU_2p5um[:,0]==20)[0],1])
print('120C/120U total from', *RatioLCLU_2p5um[np.where(RatioLCLU_2p5um[:,0]==2)[0],1], 'to', *RatioLCLU_2p5um[np.where(RatioLCLU_2p5um[:,0]==20)[0],1])
'''

CUratio = mu0*SatMag**2/(4*np.sqrt(Anisotropy*Exchange))
Cprefactor = 2*mu0*SatMag**2/(4*Anisotropy)

UCwidth = 2*np.sqrt(Exchange/Anisotropy)*1e9

print(UCwidth)
'''
"""
======================
plot data
======================
"""


ax1.plot(DW_2p5um_Ku1e4_SC[:,0], DW_2p5um_Ku1e4_SC[:,1], color = Aalto_orange, linewidth=plotlinewidth)
ax1.plot(DW_2p5um_Ku1e4_LU[:,0], DW_2p5um_Ku1e4_LU[:,1], color = Aalto_violet, linewidth=plotlinewidth)
ax1.plot(DW_2p5um_Ku1e4_LC[:,0], DW_2p5um_Ku1e4_LC[:,1],'--', color = Aalto_blue, linewidth=plotlinewidth)
ax1.plot(DW_2p5um_Ku1e4_SU[:,0], DW_2p5um_Ku1e4_SU[:,1],'--', color = Aalto_green, linewidth=plotlinewidth)


ax2.semilogy(RatioSCLU_2p5um[:,0], RatioSCLU_2p5um[:,1], color = Aalto_red, linewidth=plotlinewidth)
ax2.semilogy(RatioLCSU_2p5um[:,0], RatioLCSU_2p5um[:,1], '--', color = Aalto_turqoise, linewidth=plotlinewidth)
#ax2.semilogy(RatioSCSU_2p5um[:,0], RatioSCSU_2p5um[:,1], color = 'black', linewidth=plotlinewidth)
#ax2.semilogy(RatioLCLU_2p5um[:,0], RatioLCLU_2p5um[:,1], color = 'grey', linewidth=plotlinewidth)

"""
======================
plot design
======================
"""
#set limits
xmin = 0
xmax = 22
xint = 10
y1min = 0
y1max = 1750
y1int = 500
y2min = 15
y2max = 135
y2int = 30
y3min = -500
y3max = 6500
y3int = 2000
y3scaling = 0.001
y4min = 0
y4max = 225
y4int = 50



ax1.set_xlim(xmin, xmax)
ax2.set_xlim(xmin, xmax)

ax1.xaxis.set_ticks(np.arange(xmin, xmax, xint))
ax2.xaxis.set_ticks(np.arange(xmin, xmax, xint))

ax1.set_yticks(np.arange(0,y2max,y2int))
ax1.set_ylim(y2min,y2max)

ax2.set_yticks([1,5,10,50])
ax2.set_yticklabels(['1','5','10','50'])
ax2.set_ylim(0.8,70)


#label axes

ax1.set_xlabel('$t$ [nm] ', fontsize = label_fontsize, labelpad=xlabelpad)
ax2.set_xlabel('$t$ [nm] ', fontsize = label_fontsize, labelpad=xlabelpad)



ax1.set_ylabel(r'$\Delta$ $\phi$ [$^{\circ}$]', fontsize = label_fontsize, labelpad=ylabelpad)


ax2.set_ylabel(r'log $e_{c}/e_{u}$', fontsize = label_fontsize, labelpad=ylabelpad)
ax2.yaxis.set_label_position("right")


#tick params
ax1.tick_params(bottom=True, top=True, left=True, right=True, labelbottom=True, labeltop=False, labelleft=True, labelright=False, direction='in', labelsize=tick_fontsize)

ax2.tick_params(bottom=True, top=True, left=True, right=True, labelbottom=True, labeltop=False, labelleft=False, labelright=True, direction='in', labelsize=tick_fontsize)

ax2.minorticks_off()


"""
======================
add labels
======================
"""
#ax11.text(0.6,0.2,r'$4$ $\mu$m',fontsize=label_fontsize,transform=ax11.transAxes)

ax1.text(0.1,0.45,'60C',color = Aalto_orange,fontsize=label_fontsize,transform=ax1.transAxes)
ax1.text(0.1,0.55,'60U',color = Aalto_green,fontsize=label_fontsize,transform=ax1.transAxes)
ax1.text(0.65,0.45,'120C',color = Aalto_blue,fontsize=label_fontsize,transform=ax1.transAxes)
ax1.text(0.65,0.55,'120U',color = Aalto_violet,fontsize=label_fontsize,transform=ax1.transAxes)





ax2.text(0.5,0.23,'60C/120U',color = Aalto_red,fontsize=label_fontsize,transform=ax2.transAxes) 
ax2.text(0.5,0.7,'120C/60U',color = Aalto_turqoise,fontsize=label_fontsize,transform=ax2.transAxes)
#ax2.text(0.5,0.65,'60C/60U',color = 'black',fontsize=label_fontsize,transform=ax2.transAxes)
#ax2.text(0.4,0.35,'120C/120U',color = 'grey',fontsize=label_fontsize,transform=ax2.transAxes)


"""
add panel labels
"""
axlabels = fig.add_axes([0,0,1,1],label ='labels')
axlabels.axis('off')
axlabels.patch.set_alpha(0.0)
axlabels.text(0.0,0.92,'(a)',fontsize=label_fontsize)
axlabels.text(0.95,0.92,'(b)',fontsize=label_fontsize)
axlabels.tick_params(bottom=False, top=False, left=False, right=False, labelbottom=False, labeltop=False, labelleft=False, labelright=False)


"""
======================
show image
======================
"""
plt.savefig('t-dep_2p5um.pdf')
plt.close()
#plt.show()


